/*
 * pdwon.c : setup EINT1 for power-down and wakeup
 *   by uratan! 2008.9.9
 */
#include "common.h"
#include "LPC214x.h"
#include "irq.h"
#include "swi.h"
#include "pcon.h"

/*
 * register bitmaps
 */
/* EXTINT: External Interrupt Flag register */
#define EINT0		0x00000001
#define EINT1		0x00000002
#define EINT2		0x00000004
#define EINT3		0x00000008
/* INTWAKE: Interrupt Wakeup register */
#define EXTWAKE0	0x00000001
#define EXTWAKE1	0x00000002
#define EXTWAKE2	0x00000004
#define EXTWAKE3	0x00000008
#define USBWAKE		0x00000020
#define BODWAKE		0x00004000
#define RTCWAKE		0x00008000
/* EXTMODE: External Interrupt Mode register */
#define EINT0_EDGE	0x00000001
#define EINT1_EDGE	0x00000002
#define EINT2_EDGE	0x00000004
#define EINT3_EDGE	0x00000008
/* EXTPOLAR: External Interrupt Polarity register */
#define EINT0_RISING	0x00000001
#define EINT1_RISING	0x00000002
#define EINT2_RISING	0x00000004
#define EINT3_RISING	0x00000008

/*
 * functions
 */
void pdown_init(void);
void power_down(void);
void pdown_sys_pll_init(void);

/*
 *
 */
void pdown_init(void)
{
	PINSEL0 |= 0x20000000;		/* set P0.14 as EINT1 */
// use P0.7/EINT2 with UZK

	INTWAKE = 0;				/* no INTx will wake up the processor */
	EXTMODE = EINT1_EDGE;		/* INT1 edge trigger */
	EXTPOLAR = 0;				/* INT1 is falling edge by default */
		/*
		 * no need to use intr-handler to wake up operation
		 */
	return;
}

/*
 *
 */
void power_down(void)
{
	u_long imf;
	volatile u_long t;

	imf = IntDisable();				/* disable interrupts */

	/*
	 * wait for UART job finished (about 1.3msec with -Os, 60MHz)
	 */
	for(t=0; t<5000; t++)
		;

	/*
	 * set wake up source
	 */
	INTWAKE = EXTWAKE1;			/* INT1 will wake up the processor */

	/*
	 * then try to Power Down
	 */
	PCON = PC_PD;
		/*
		 * now in power down mode !
		 */

	/*
	 * I have waked up now
	 */
	INTWAKE = 0;				/* no INTx will wake up the processor */

	/*
	 * re-initial PLL for system clock
	 */
	pdown_sys_pll_init();

/*
 * clear some status (recv-buff, etc.)
 */

	/*
	 * recover previous state
	 */
	IntRestore(imf);				/* enable interrupts */
}

/*
 *
 */
void pdown_sys_pll_init(void)
{
	/*
	 * initial PLL for system clock
	 */
	PLLCON = 1;				/* enable PLL */

#if (Fcco / Fcclk) == 2
	PLLCFG = ((Fcclk / Fosc) - 1) | (0 << 5);
#elif (Fcco / Fcclk) == 4
	PLLCFG = ((Fcclk / Fosc) - 1) | (1 << 5);			/* <== */
#elif (Fcco / Fcclk) == 8
	PLLCFG = ((Fcclk / Fosc) - 1) | (2 << 5);
#elif (Fcco / Fcclk) == 16
	PLLCFG = ((Fcclk / Fosc) - 1) | (3 << 5);
#else
#error invalid PLLCFG
#endif
	PLLFEED = 0xaa;
	PLLFEED = 0x55;

	/*
	 * wait for PLL lock
	 */
	while((PLLSTAT & (1<<10)) == 0)
		;

	PLLCON = 3;				/* enable & connect PLL */
	PLLFEED = 0xaa;
	PLLFEED = 0x55;
}
